@@ -7,19 +7,19 @@ use diagnostics;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( auction );
-our $VERSION = '0.50';
+our $VERSION = '0.60';
 
 #Variables global to the package	
 my $maximize_total_benefit;
 my $matrix_spaces;     # used to print messages on the screen
 my $decimals;          # the number of digits after the decimal point
 my $max_matrix_value;
-my $need_transpose = 0;
+my $need_transpose;
 my ( $array1_size, $array2_size, $min_size, $max_size, $original_max_size );
 my ( $inicial_price, $iter_count_global, $iter_count_local, $epsilon_scaling );
-my ( %index_correlation, %assignned_object, %assignned_person, %price_object, %objects_desired_by_this );
+my ( %index_correlation, %assignned_object, %assignned_person, %price_object, %objects_desired_by_this, %filtered_list, %locked_person );
 
-sub auction { #							=> default values
+sub auction { #						=> default values
 	my %args = ( matrix_ref				=> undef,     # reference to array: matrix N x M			                                     
 				maximize_total_benefit  => 0,         # 0: minimize_total_benefit ; 1: maximize_total_benefit
 				inicial_stepsize        => undef,     # auction algorithm terminates with a feasible assignment if the problem data are integer and stepsize < 1/min(N,M)				
@@ -31,9 +31,12 @@ sub auction { #							=> default values
 	$max_matrix_value = 0;
 	$iter_count_global = 0;
 	$epsilon_scaling = 0;
+	$need_transpose = 0;
 	%index_correlation = ();
 	%price_object = ();
-	%objects_desired_by_this = ();	
+	%objects_desired_by_this = ();
+	%filtered_list = ();
+	%locked_person = ();
 	
 	my @matrix_input = @{$args{matrix_ref}};          # Input: Reference to the input matrix (NxM) = $min_size x $max_size
    
@@ -79,6 +82,7 @@ sub auction { #							=> default values
 	
 	$inicial_price    = $args{inicial_price};
 	$price_object{$_} = $inicial_price for ( 0 .. $max_size - 1 );
+	#$price_object{$_} = sprintf( "%.0f", rand($min_size) ) for ( 0 .. $max_size - 1 ); # random values for initial prices
    
 	# inicial epsilon value
 	my $epsilon = ($max_matrix_value/2) * exp ( -1 * $max_size/$min_size );  # exp (1) = e = 2.71828182845905
@@ -442,8 +446,11 @@ sub auctionRound {
 
 			my ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );
 			my ( $Opt01ValForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI ) = ( -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value) );
-			my $BadValForPersonI = 10 * exp ($max_matrix_value);
-			my ( $bid01ForPersonI, $bid02ForPersonI );			
+			
+			my ( $Opt01ObjForPersonI_new_list, $Opt02ObjForPersonI_new_list, $Opt03ObjForPersonI_new_list );
+			my ( $Opt01ValForPersonI_new_list, $Opt02ValForPersonI_new_list, $Opt03ValForPersonI_new_list ) = ( -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value) );
+
+			my ( $bid01ForPersonI, $bid02ForPersonI );
 			my %current_value;
 			my @updated_price;
 			
@@ -456,14 +463,14 @@ sub auctionRound {
 			
 			# There is at least one object in the @objects_with_greater_benefits list whose price is updated ? use old list : generate new list;
 			
-			for my $object ( @objects_with_greater_benefits ){ # use old list
-			
+			for my $object ( @objects_with_greater_benefits )  # use old list	
+			{				
 				my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object];
 				$current_value{$object} = $matrix_value - $price_object{$object};
 				
 				push @updated_price, $object if ( $objects_desired_by_this{$person}{$object} == $current_value{$object} );
 
-				if ( $current_value{$object} > $Opt01ValForPersonI )
+				if ( $current_value{$object} > $Opt01ValForPersonI ) # search for the best 3 objects
 				{
 					$Opt03ValForPersonI = $Opt02ValForPersonI;
 					$Opt03ObjForPersonI = $Opt02ObjForPersonI;
@@ -483,48 +490,141 @@ sub auctionRound {
 				{
 					$Opt03ValForPersonI = $current_value{$object};
 					$Opt03ObjForPersonI = $object;
-				}
+				}								
 
 				if ( $verbose >= 8 ){
 					printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f <old list> \n", $person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object} ;
 				}
 			}
 
-			if ( not @updated_price ){ # if all prices are outdated
-		
-				for my $object ( 0 .. $max_size - 1 ) # generate new list
+			if ( not @updated_price and not defined $locked_person{$person} )  # if all prices are outdated
+			{				
+				if ( $epsilon_scaling <= 2 )
 				{
-					next if ( defined $current_value{$object} );
+					for my $object ( 0 .. $max_size - 1 ) # generate new list
+					{
+						next if ( defined $current_value{$object} );
 					
-					my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object];				
-					$current_value{$object} = $matrix_value - $price_object{$object};
+						my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object];				
+						$current_value{$object} = $matrix_value - $price_object{$object};
 				
-					if ( $current_value{$object} > $Opt01ValForPersonI )
-					{
-						$Opt03ValForPersonI = $Opt02ValForPersonI;
-						$Opt03ObjForPersonI = $Opt02ObjForPersonI;
-						$Opt02ValForPersonI = $Opt01ValForPersonI;
-						$Opt02ObjForPersonI = $Opt01ObjForPersonI;
-						$Opt01ValForPersonI = $current_value{$object};
-						$Opt01ObjForPersonI = $object;
-					}
-					elsif ( $current_value{$object} > $Opt02ValForPersonI )
-					{
-						$Opt03ValForPersonI = $Opt02ValForPersonI;
-						$Opt03ObjForPersonI = $Opt02ObjForPersonI;
-						$Opt02ValForPersonI = $current_value{$object};
-						$Opt02ObjForPersonI = $object;
-					}
-					elsif ( $current_value{$object} > $Opt03ValForPersonI )
-					{
-						$Opt03ValForPersonI = $current_value{$object};
-						$Opt03ObjForPersonI = $object;
-					}
+						if ( $current_value{$object} > $Opt01ValForPersonI_new_list ) # to find the best 3 objects in the complementary subset
+						{
+							$Opt03ValForPersonI_new_list = $Opt02ValForPersonI_new_list;
+							$Opt03ObjForPersonI_new_list = $Opt02ObjForPersonI_new_list;
+							$Opt02ValForPersonI_new_list = $Opt01ValForPersonI_new_list;
+							$Opt02ObjForPersonI_new_list = $Opt01ObjForPersonI_new_list;
+							$Opt01ValForPersonI_new_list = $current_value{$object};
+							$Opt01ObjForPersonI_new_list = $object;
+						}
+						elsif ( $current_value{$object} > $Opt02ValForPersonI_new_list )
+						{
+							$Opt03ValForPersonI_new_list = $Opt02ValForPersonI_new_list;
+							$Opt03ObjForPersonI_new_list = $Opt02ObjForPersonI_new_list;
+							$Opt02ValForPersonI_new_list = $current_value{$object};
+							$Opt02ObjForPersonI_new_list = $object;
+						}
+						elsif ( $current_value{$object} > $Opt03ValForPersonI_new_list )
+						{
+							$Opt03ValForPersonI_new_list = $current_value{$object};
+							$Opt03ObjForPersonI_new_list = $object;
+						}
 
-					if ( $verbose >= 8 ){
-						printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f \n", $person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object};
-					}				
-				}	
+						if ( $verbose >= 8 ){
+							printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f ; \$Opt01ValForPersonI_new_list = %12.5f\n", $person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object}, $Opt01ValForPersonI_new_list;
+						}				
+					}
+				} 
+				elsif ( $epsilon_scaling >= 3 and $epsilon_scaling <= 6 )
+				{
+					for my $object ( 0 .. $max_size - 1 ) # generate new list
+					{
+						next if ( defined $current_value{$object} );
+					
+						my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object];				
+						$current_value{$object} = $matrix_value - $price_object{$object};
+				
+						if ( $current_value{$object} > $Opt01ValForPersonI_new_list ) # to find the best 2 objects in the complementary subset
+						{
+							$Opt02ValForPersonI_new_list = $Opt01ValForPersonI_new_list;
+							$Opt02ObjForPersonI_new_list = $Opt01ObjForPersonI_new_list;
+							$Opt01ValForPersonI_new_list = $current_value{$object};
+							$Opt01ObjForPersonI_new_list = $object;
+						}
+						elsif ( $current_value{$object} > $Opt02ValForPersonI_new_list )
+						{
+							$Opt02ValForPersonI_new_list = $current_value{$object};
+							$Opt02ObjForPersonI_new_list = $object;
+						}
+
+						if ( $verbose >= 8 ){
+							printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f ; \$Opt01ValForPersonI_new_list = %12.5f\n", $person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object}, $Opt01ValForPersonI_new_list;
+						}				
+					}
+				}
+				elsif ( $epsilon_scaling >= 7 ) # in most cases, need only one object
+				{
+					for my $object ( 0 .. $max_size - 1 ) # generate new list
+					{
+						next if ( defined $current_value{$object} );
+					
+						my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object];				
+						$current_value{$object} = $matrix_value - $price_object{$object};
+				
+						if ( $current_value{$object} > $Opt01ValForPersonI_new_list ) # to find the best object in the complementary subset
+						{
+							$Opt01ValForPersonI_new_list = $current_value{$object};
+							$Opt01ObjForPersonI_new_list = $object;
+						}
+
+						if ( $verbose >= 8 ){
+							printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f ; \$Opt01ValForPersonI_new_list = %12.5f\n", $person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object}, $Opt01ValForPersonI_new_list;
+						}				
+					}
+				}			
+				
+				# to find the best 3 out of 6 objects
+
+				if ( $Opt01ValForPersonI_new_list > $Opt01ValForPersonI )
+				{
+					$Opt03ValForPersonI = $Opt02ValForPersonI;
+					$Opt03ObjForPersonI = $Opt02ObjForPersonI;
+					$Opt02ValForPersonI = $Opt01ValForPersonI;
+					$Opt02ObjForPersonI = $Opt01ObjForPersonI;
+					$Opt01ValForPersonI = $Opt01ValForPersonI_new_list;
+					$Opt01ObjForPersonI = $Opt01ObjForPersonI_new_list;
+				}
+				elsif ( $Opt01ValForPersonI_new_list > $Opt02ValForPersonI )
+				{
+					$Opt03ValForPersonI = $Opt02ValForPersonI;
+					$Opt03ObjForPersonI = $Opt02ObjForPersonI;
+					$Opt02ValForPersonI = $Opt01ValForPersonI_new_list;
+					$Opt02ObjForPersonI = $Opt01ObjForPersonI_new_list;
+				}
+				elsif ( $Opt01ValForPersonI_new_list > $Opt03ValForPersonI )
+				{
+					$Opt03ValForPersonI = $Opt01ValForPersonI_new_list;
+					$Opt03ObjForPersonI = $Opt01ObjForPersonI_new_list;
+				}
+
+				if ( $Opt02ValForPersonI_new_list > $Opt02ValForPersonI )
+				{
+					$Opt03ValForPersonI = $Opt02ValForPersonI;
+					$Opt03ObjForPersonI = $Opt02ObjForPersonI;
+					$Opt02ValForPersonI = $Opt02ValForPersonI_new_list;
+					$Opt02ObjForPersonI = $Opt02ObjForPersonI_new_list;
+				}
+				elsif ( $Opt02ValForPersonI_new_list > $Opt03ValForPersonI )
+				{
+					$Opt03ValForPersonI = $Opt02ValForPersonI_new_list;
+					$Opt03ObjForPersonI = $Opt02ObjForPersonI_new_list;
+				}
+
+				if ( $Opt03ValForPersonI_new_list > $Opt03ValForPersonI )
+				{
+					$Opt03ValForPersonI = $Opt03ValForPersonI_new_list;
+					$Opt03ObjForPersonI = $Opt03ObjForPersonI_new_list;
+				}				
 			}
 			
 			$bid01ForPersonI = $Opt01ValForPersonI - $Opt02ValForPersonI + $epsilon;
@@ -532,26 +632,43 @@ sub auctionRound {
 			# Stores the bidding info for future use
 			$info{$Opt01ObjForPersonI}{$bid01ForPersonI} = $person if ( not defined $info{$Opt01ObjForPersonI}{$bid01ForPersonI} ); # get only one person with this bid			
 
-			if ( $max_size >= 10 ){
+			if ( $max_size >= 6 ){
 			
 				$bid02ForPersonI = $Opt02ValForPersonI - $Opt03ValForPersonI + $epsilon; # calculate $bid02ForPersonI to try to predict the values of the next round
 				$bid02ForPersonI = 2 * $epsilon if ( $Opt01ValForPersonI > ($Opt02ValForPersonI + $epsilon) or $Opt02ValForPersonI > ($Opt03ValForPersonI + $epsilon) );
-			
+
 				$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );				
 					
-				if ( not @updated_price ){ # if all prices are outdated						
-
+				if ( not @updated_price ){ # if all prices are outdated
+					
 					$objects_desired_by_this{$person}{$_} = $current_value{$_} for ( @objects_with_greater_benefits ); # update current values for all objects in the old list
 					
-					if ( $epsilon_scaling == 6 and $iter_count_local <= $epsilon_scaling and @objects_with_greater_benefits > $epsilon_scaling )
+					if ( $filtered_list{$person} )
+					{
+						$Opt01ValForPersonI_new_list = $locked_person{$person} if ( defined $locked_person{$person} );
+						
+						# Lock or unlock the old list after reviewing the quality of the list objects
+						if ( ($Opt03ValForPersonI - $Opt01ValForPersonI_new_list) > 4 * $epsilon )
+						{
+							$Opt01ValForPersonI_new_list -= $epsilon;
+							$locked_person{$person} = $Opt01ValForPersonI_new_list; # Lock the old list
+							printf "<> PersonI = %3s ; dif %.5f ( \$Opt03ValForPersonI %.5f - \$Opt01ValForPersonI_new_list %.5f ) > (4 * \$epsilon) %.5f ; (Max_CurVal %.5f) *** locked person *** \n", $person, $Opt03ValForPersonI - $Opt01ValForPersonI_new_list, $Opt03ValForPersonI, $Opt01ValForPersonI_new_list, 4 * $epsilon, $locked_person{$person} if ( $verbose >= 8 );
+						} elsif ( defined $locked_person{$person} ) {							
+							printf "<> PersonI = %3s ; dif %.5f ( \$Opt03ValForPersonI %.5f - \$Opt01ValForPersonI_new_list %.5f ) <= (4 * \$epsilon) %.5f ; (Max_CurVal %.5f) *** unlocked person *** \n", $person, $Opt03ValForPersonI - $Opt01ValForPersonI_new_list, $Opt03ValForPersonI, $Opt01ValForPersonI_new_list, 4 * $epsilon, $locked_person{$person} if ( $verbose >= 8 );
+							delete $filtered_list{$person}; # filter again
+							delete $locked_person{$person}; # Unlock the old list
+						}
+					}
+					elsif ( $epsilon_scaling >= 6 ) # filter the old list
 					{
 						for my $object ( @objects_with_greater_benefits ){
-							$BadValForPersonI = $current_value{$object} if ( $current_value{$object} < $BadValForPersonI );
+							$filtered_list{$person} = 1;
+							next unless ( ($Opt01ValForPersonI - $current_value{$object}) > 1 * $min_size * $epsilon );
+							delete $objects_desired_by_this{$person}{$object}; # delete objects of little benefit from the old list 							
+							printf "<> PersonI = %3s ; *** delete the object %3s from the old list *** \n", $person, $object if ( $verbose >= 8 );
 						}
-
-						delete $objects_desired_by_this{$person} if ( ($Opt01ValForPersonI - $BadValForPersonI) > 2 * $min_size * $epsilon ); # delete old list
-					}
-
+					}					
+					
 					$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = $current_value{$Opt01ObjForPersonI}; # information about the most desired objects
 					$objects_desired_by_this{$person}{$Opt02ObjForPersonI} = $current_value{$Opt02ObjForPersonI} if (defined $Opt02ObjForPersonI);
 					$objects_desired_by_this{$person}{$Opt03ObjForPersonI} = $current_value{$Opt03ObjForPersonI} if (defined $Opt03ObjForPersonI);					
@@ -562,11 +679,9 @@ sub auctionRound {
 				my @old_list = sort { $a <=> $b } @objects_with_greater_benefits;
 				my @new_list = sort { $a <=> $b } keys %{$objects_desired_by_this{$person}};
 				my @best_3_objects = ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );
-				my $add_msg = " ";
-				$add_msg = sprintf "; \$BadValForPersonI = %.5f", $BadValForPersonI if ( $epsilon_scaling == 6 and $iter_count_local <= $epsilon_scaling and @objects_with_greater_benefits > $epsilon_scaling );
 
 				printf "<> PersonI = %3s ; %3s objects desired by this person (old list) = (@old_list) ; objects whose current values are still updated = (@updated_price) : %2s >= 1 ? \n", $person, scalar @old_list, scalar @updated_price;
-				printf "<> PersonI = %3s ; %3s objects desired by this person (new list) = (@new_list) ; \@best_3_objects = (@best_3_objects) $add_msg\n", $person, scalar @new_list if ( defined $Opt03ObjForPersonI );
+				printf "<> PersonI = %3s ; %3s objects desired by this person (new list) = (@new_list) ; \@best_3_objects = (@best_3_objects)\n", $person, scalar @new_list if ( defined $Opt03ObjForPersonI );
 				printf "<> PersonI = %3s chose ObjectJ = %3s ; \$bid01ForPersonI %10.5f = \$Opt01ValForPersonI %.5f - \$Opt02ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt01ObjForPersonI, $bid01ForPersonI, $Opt01ValForPersonI, $Opt02ValForPersonI, $epsilon;
 				printf "<> PersonI = %3s chose ObjectJ = %3s ; \$bid02ForPersonI %10.5f = \$Opt02ValForPersonI %.5f - \$Opt03ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt02ObjForPersonI, $bid02ForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $epsilon if ( defined $bid02ForPersonI );
 			}
@@ -657,9 +772,10 @@ __END__
 
  use Algorithm::Bertsekas qw(auction);
  
- Example 1: Minimize the total benefit.
- my @array1 = ( 22, 15, 98,  1 );
- my @array2 = ( 72, 99, 29, 88, 12, 26, 41 );
+ Example 1: Find the nearest neighbor, Minimize the total benefit.
+
+ my @array1 = ( 893, 401, 902, 576, 767, 917, 76, 464, 124, 207, 125, 530 );
+ my @array2 = ( 161, 559, 247, 478, 456 );
 
  my @input_matrix;
  for my $i ( 0 .. $#array1 ){
@@ -671,53 +787,74 @@ __END__
    }
    push @input_matrix, \@weight_function;
  }
-
-      72 99 29 88 12 26 41
-
- 22 [ 50 77  7 66 10  4 19 ]
- 15 [ 57 84 14 73  3 11 26 ]
- 98 [ 26  1 69 10 86 72 57 ]
-  1 [ 71 98 28 87 11 25 40 ]
   
+       161 559 247 478 456
+
+ 893 [ 732 334 646 415 437 ]
+ 401 [ 240 158 154  77  55 ]
+ 902 [ 741 343 655 424 446 ]
+ 576 [ 415  17 329  98 120 ]
+ 767 [ 606 208 520 289 311 ]
+ 917 [ 756 358 670 439 461 ]
+  76 [  85 483 171 402 380 ]
+ 464 [ 303  95 217  14   8 ]
+ 124 [  37 435 123 354 332 ]
+ 207 [  46 352  40 271 249 ]
+ 125 [  36 434 122 353 331 ]
+ 530 [ 369  29 283  52  74 ]
+ 
  my ( $optimal, $assignment_ref, $output_index_ref ) = auction( matrix_ref => \@input_matrix, maximize_total_benefit => 0, verbose => 5 );
   
  Objective: to Minimize the total benefit
- Number of left nodes: 4
- Number of right nodes: 7
- Number of edges: 28
+ Number of left nodes: 12
+ Number of right nodes: 5
+ Number of edges: 60
 
  Solution:
- Optimal assignment: sum of values = 30
- Feasible assignment condition: stepsize = 0.2 < 1/4 = 0.25
- Number of iterations: 15
+ Optimal assignment: sum of values = 153
+ Feasible assignment condition: stepsize = 0.1667 < 1/5 = 0.2
+ Number of iterations: 50
 
- row index    = [ 0  1  2  3  4  5  6 ]
- column index = [ 2  5  1  4  0  6  3 ]
- matrix value = [ 7 11  1 11          ]
+ row index    = [  0   1   2   3   4   5   6   7   8   9  10  11 ]
+ column index = [  9   8  10   1   5  11   7   4   6   2   0   3 ]
+ matrix value = [             17               8      40  36  52 ]
 
- modified matrix 4 x 7:
- [ 48   21   91** 32   88   94   79  ]
- [ 41   14   84   25   95   87** 72  ]
- [ 72   97** 29   88   12   26   41  ]
- [ 27    0   70   11   87** 73   58  ]
+ modified matrix 5 x 9:
+ [ 516   341   150   671   453   719   710   720** 387  ]
+ [ 598   739** 548   273   661   321   404   322   727  ]
+ [ 602   427   236   585   539   633   716** 634   473  ]
+ [ 679   658   467   354   742   402   485   403   704**]
+ [ 701   636   445   376   748** 424   507   425   682  ]
 
- original matrix 4 x 7 with solution:
- [ 50   77    7** 66   10    4   19  ]
- [ 57   84   14   73    3   11** 26  ]
- [ 26    1** 69   10   86   72   57  ]
- [ 71   98   28   87   11** 25   40  ]
+ original matrix 12 x 5 with solution:
+ [ 732   334   646   415   437  ]
+ [ 240   158   154    77    55  ]
+ [ 741   343   655   424   446  ]
+ [ 415    17** 329    98   120  ]
+ [ 606   208   520   289   311  ]
+ [ 756   358   670   439   461  ]
+ [  85   483   171   402   380  ]
+ [ 303    95   217    14     8**]
+ [  37   435   123   354   332  ]
+ [  46   352    40** 271   249  ]
+ [  36** 434   122   353   331  ]
+ [ 369    29   283    52**  74  ]
 
  Pairs (in ascending order of matrix values):
-   indices ( 2, 1 ), matrix value =  1 ; sum of values =  1
-   indices ( 0, 2 ), matrix value =  7 ; sum of values =  8
-   indices ( 1, 5 ), matrix value = 11 ; sum of values = 19
-   indices ( 3, 4 ), matrix value = 11 ; sum of values = 30
-   indices ( 4, 0 ), matrix value =    ; sum of values = 30
-   indices ( 5, 6 ), matrix value =    ; sum of values = 30
-   indices ( 6, 3 ), matrix value =    ; sum of values = 30  
+   indices (  7,  4 ), matrix value =   8 ; sum of values =     8
+   indices (  3,  1 ), matrix value =  17 ; sum of values =    25
+   indices ( 10,  0 ), matrix value =  36 ; sum of values =    61
+   indices (  9,  2 ), matrix value =  40 ; sum of values =   101
+   indices ( 11,  3 ), matrix value =  52 ; sum of values =   153
+   indices (  0,  9 ), matrix value =     ; sum of values =   153
+   indices (  1,  8 ), matrix value =     ; sum of values =   153
+   indices (  2, 10 ), matrix value =     ; sum of values =   153
+   indices (  4,  5 ), matrix value =     ; sum of values =   153
+   indices (  5, 11 ), matrix value =     ; sum of values =   153
+   indices (  6,  7 ), matrix value =     ; sum of values =   153
+   indices (  8,  6 ), matrix value =     ; sum of values =   153
   
  Example 2: Maximize the total benefit.
- Alternatively, we can define the matrix with its elements:
  
  my $N = 10;
  my $M = 10;
@@ -732,6 +869,8 @@ __END__
    }
    push @input_matrix, \@weight_function;
  }
+ 
+ Alternatively, we can define the matrix with its elements:
 
  my @input_matrix = ( 
  [  84,  94,  75,  56,  66,  95,  39,  53,  73,   4 ],
@@ -848,7 +987,7 @@ __END__
 =head1 AUTHOR
 
     Claudio Fernandes de Souza Rodrigues
-	April 11, 2018
+	April 13, 2018
 	Sao Paulo, Brasil
 	claudiofsr@yahoo.com
 	
